package entities;

import java.lang.reflect.Constructor;
import java.util.List;
import java.util.Set;
import java.util.logging.ConsoleHandler;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.ManyToOne;
import javax.persistence.TypedQuery;

import definitions.ActiveStatement;
import definitions.Findable;
import definitions.Language;

@Entity
public class BlockStatement extends Statement implements Findable,ActiveStatement{
	
	private static transient final Logger LOG=Logger.getLogger(BlockStatement.class.getName());
	static{
		
			LOG.setLevel(Level.ALL);
			ConsoleHandler console=new ConsoleHandler();
			console.setLevel(Level.ALL);
	}
	
	
	private User enemy;

	@ManyToOne
	@Column
	public User getEnemy() {
		return this.enemy;
	}
	public BlockStatement(){
		super();
	}

	public BlockStatement(User enemy) {
		super();
		this.enemy = enemy;
	}

	public void setEnemy(User enemy) {
		this.enemy = enemy;
	}

	@Override
	String toString(Language lang) {
		
		String sentence=null;
		
		switch (lang){
		
				case English: sentence="Block "+this.enemy.toString(Language.English);
				break;
		}
		// TODO Auto-generated method stub
		return sentence;
	}

	
	public Constructor getConstructorForFactory() {
		
		Constructor con =null;
		
		try {
			
			con=BlockStatement.class.getConstructor(User.class);
			
		} catch (NoSuchMethodException | SecurityException e) {
			
			LOG.severe("FollowStatement could not supply constructor for factory because "+e.getMessage());
		}
		return con;
	}

	@Override
	public void doAnActionOnANewBeliever(Subject believer) {
		
	Set<User> blockList=believer.getBlockSet();
	
		blockList.add(enemy);
	}
	
	
		// TODO Auto-generated method stub
		
	

	@Override
	public void doAnActionOnAnExBeliever(Subject believer) {
		
		Set<User> blockSet=believer.getBlockSet();
		
		blockSet.remove(enemy);
		
		
		// TODO Auto-generated method stub
		
	}

	@Override
	public String getFindingQLString() {
		// TODO Auto-generated method stub
		return "select b from BlockStatement b where b.enemy.id=?0";
	}

	@Override
	public TypedQuery<LogicalObject> prepareQuery(TypedQuery<LogicalObject> query, Object[] args) {
		// TODO Auto-generated method stub
		User enemy=(User) args[0];
		
		return query.setParameter(0,enemy.getId());
		
		
	}

}
