package entities;

import java.lang.reflect.Constructor;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.ConsoleHandler;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.persistence.Entity;
import javax.persistence.ManyToOne;
import javax.persistence.TypedQuery;

import services.Factory;

import definitions.Findable;
import definitions.Inferable;
import definitions.Language;

@Entity
public class Conjunction extends Statement implements Inferable, Findable{
	
	
	
	@ManyToOne
	public Statement getConjunct1() {
		return conjunct1;
	}

	public void setConjunct1(Statement conjunct1) {
		this.conjunct1 = conjunct1;
	}

	@ManyToOne
	public Statement getConjunct2() {
		return conjunct2;
	}

	public Conjunction(Statement conjunct1, Statement conjunct2) {
		super();
		this.conjunct1 = conjunct1;
		this.conjunct2 = conjunct2;
	}

	public void setConjunct2(Statement conjunct2) {
		this.conjunct2 = conjunct2;
	}

	

	private static transient final Logger LOG=Logger.getLogger(Conjunction.class.getName());
	static{
		
			LOG.setLevel(Level.ALL);
			ConsoleHandler console=new ConsoleHandler();
			console.setLevel(Level.ALL);
		
		
	}
	
	public Conjunction() {
		
		
		super();
		
		// TODO Auto-generated constructor stub
	}

	private Statement conjunct1;
	
	private Statement conjunct2;

	@Override
	public List<Statement> infer(Factory factory) {
		
		List<Statement> inferences=new ArrayList<Statement>();
		inferences.add(conjunct1);
		inferences.add(conjunct2);
		
		return inferences;
	}

	

	@Override
	String toString(Language lang) {
	
		
		String sentence=null;
		
		switch(lang){
		
				case English: sentence="["+conjunct1.toString(lang)+"] and ["+conjunct2.toString(lang)+"]";
				break;
		}
		
		return sentence;
	}

	
	

	public Constructor getConstructorForFactory() {
		
		Constructor con=null;
		try {
			con=Conjunction.class.getConstructor(Statement.class, Statement.class);
		} catch (NoSuchMethodException | SecurityException e) {
			
			
			LOG.severe("Conjunction could not supply constructor for factory!");
		}
		return con;
	}

	@Override
	public String getFindingQLString() {
		// TODO Auto-generated method stub
		return "select c from Conjunction c where c.conjunct1.id=?0 and c.conjunct2.id=?1";
	}

	@Override
	public TypedQuery<LogicalObject> prepareQuery(TypedQuery<LogicalObject> query, Object... args) {
		
		Statement conjunct1=(Statement) args[0];
		
		Statement conjunct2=(Statement) args[1];
		
		
			query.setParameter(0,conjunct1.getId());
			
			query.setParameter(1,conjunct2.getId());
			
			return query;
		
	}

}
