package entities;

import java.lang.reflect.Constructor;
import java.util.logging.ConsoleHandler;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jdo.annotations.Index;
import javax.persistence.ManyToOne;
import javax.persistence.MappedSuperclass;
import javax.persistence.TypedQuery;

import services.Factory;


import definitions.ActiveStatement;
import definitions.Findable;
import definitions.Language;
import entities.Statement;



@MappedSuperclass
public abstract class Implication extends Statement implements ActiveStatement{
	
	private static transient final Logger LOG=Logger.getLogger(Implication.class.getName());
	static{
		
			LOG.setLevel(Level.ALL);
			ConsoleHandler console=new ConsoleHandler();
			console.setLevel(Level.ALL);
		
		
	}
	
	public Implication(){
		super();
	}
	
	
	


	protected Statement assumption;
	
	protected Statement conclusion;
	

	public Implication(Statement assumption, Statement conclusion) {
		
		this.assumption=assumption;
		this.conclusion=conclusion;
}
	@Index
	@ManyToOne
	public Statement getAssumption() {
		return assumption;
	}
	@Index
	@ManyToOne
	public Statement getConclusion() {
		return conclusion;
	}

	

	public void setAssumption(Statement assumption) {
		this.assumption = assumption;
	}

	public void setConclusion(Statement conclusion) {
		this.conclusion = conclusion;
	}
	


public  Constructor getConstructorForFactory() {
	Constructor con=null;
	
	try {
		con=this.getClass().getConstructor(Statement.class,Statement.class);
	} catch (NoSuchMethodException | SecurityException e) {
		
		LOG.severe("Implication could not give constructor to the factory! because "+e.getMessage());
	}
	return con;
}


public String getFindingQLString() {
	// 
	return "select i from Implication i where i.assumption.id=?0 and i.conclusion.id=?1";
}

	
	
	

	
	
	public  TypedQuery<LogicalObject> prepareQuery(TypedQuery<LogicalObject> query,Object...args){
		
		
		Statement assumption=(Statement) args[0];
		Statement conclusion=(Statement) args[1];
		
		query.setParameter(0,assumption.getId()).setParameter(1, conclusion.getId());
		
		return query;
		
		
		
		// TODO Auto-generated method stub
		
	}
	
public void doAnActionOnANewBeliever(Subject believer) {
	
	believer.getImplications().add(this);
}
	
	public void doAnActionOnAnExBeliever(Subject believer) {
		
		believer.getImplications().remove(this);
	}

	
	String toString(Language lang) {
		String sentence=null;
		
		switch(lang){
		case English: sentence="if ["+assumption+"] than ["+conclusion+"]";
		break;
		
		case Hebrew: sentence="אם "+assumption+"  אז"+conclusion;
		}
		
		return sentence;
	}
	
	
	
	
}

