package entities;
import java.lang.reflect.Constructor;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.logging.ConsoleHandler;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.persistence.Basic;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.Table;

import definitions.Language;





@Entity
@Table
public class Subject extends LogicalObject{
	
	transient private static  final Logger LOG=Logger.getLogger(Subject.class.getName());
	static{
		
			LOG.setLevel(Level.ALL);
			ConsoleHandler console=new ConsoleHandler();
			console.setLevel(Level.ALL);
		
		
	}
	
	
	public Subject(User user){
		this();
		this.setUser(user);
	}
	
	public void setBelieves(Map<Statement, References> believes) {
		this.believes = believes;
	}
	
	private String name;
	
	private User user;
	@ManyToOne(cascade={CascadeType.PERSIST,CascadeType.MERGE})
	public User getUser() {
		return user;
	}
	public void setUser(User user) {
		this.user = user;
	}
	@Basic	
	public String getName() {
		return name;
	}
	public void setName(String name) {
		this.name = name;
	}

		//Statement that the subject believe in
		private Map<Statement, References> believes;
		
		
		// implications waiting to infer conclusions
		private List<Implication> implications;
		// negations waiting to infer contradictions
		
		//contradictions the subject should solve
		 private Map<Statement, References> contradictions;

		
		
		//followers the subject should send statements
		private Set<Subject>followers;
		
		
		//implications which the subject able to agree with them and with their assumptions but disagree with their conclusions
		private List<Implication> challenges;
		
		private List<Statement>trash;
		
		private Set<User>blockSet;
		
		//statements needs to remove deductions when removing the statement
		private Map<Statement, Statements> removeIndex;
		
		//When conclusion is agreed the challenge need to be removed
				private Map<Statement, Implication> conclusionToChallengeIndex;
				
				//when assumption is blocked or disagreed the challenge need to be removed
				private Map<Statement, Implication> assumptionToChallengeIndex;
				
				
		
		 

				@ManyToMany
				public Map<Statement, Implication> getConclusionToChallengeIndex() {
					return conclusionToChallengeIndex;
				}

				public void setConclusionToChallengeIndex(
						Map<Statement, Implication> conclusionToChallengeIndex) {
					this.conclusionToChallengeIndex = conclusionToChallengeIndex;
				}

				@ManyToMany
				public Map<Statement, Implication> getAssumptionToChallengeIndex() {
					return assumptionToChallengeIndex;
				}

				public void setAssumptionToChallengeIndex(
						Map<Statement, Implication> assumptionToChallengeIndex) {
					this.assumptionToChallengeIndex = assumptionToChallengeIndex;
				}

		@ManyToMany
		public Set<User> getBlockSet() {
			return blockSet;
		}
		public void setBlockSet(Set<User> blockSet) {
			this.blockSet = blockSet;
		}
		public List<Implication> getChallenges() {
			return challenges;
		}
		public void setChallenges(List<Implication> challenges) {
			this.challenges = challenges;
		}
		public List<Statement> getTrash() {
			return trash;
		}
		public void setTrash(List<Statement> trash) {
			this.trash = trash;
		}
		@ManyToMany
		public Set<Subject> getFollowers() {
			return followers;
		}
		public void setFollowers(Set<Subject> followers) {
			this.followers = followers;
		}
		@ManyToMany
		public Map<Statement, References> getContradictions() {
			return contradictions;
		}
		public void setContradictions(Map<Statement, References> contradictions) {
			this.contradictions = contradictions;
		}
		@Column
		@Basic
		@ManyToMany(cascade={CascadeType.PERSIST,CascadeType.MERGE,CascadeType.REFRESH})
	public Map<Statement,References> getBelieves() {
			return believes;
		}
	@ManyToMany
	public List<Implication> getImplications() {
		return implications;
	}
	public void setImplications(List<Implication> implications) {
		this.implications = implications;
	}
	@ManyToMany
	public Map<Statement, Statements> getRemoveIndex() {
		return removeIndex;
	}





	public void setRemoveIndex(Map<Statement, Statements> removeIndex) {
		this.removeIndex = removeIndex;
	}
	

	public Subject(){
		this.believes=new HashMap<Statement,References>();
		
		this.implications=new ArrayList<Implication>();
		this.contradictions=new HashMap<Statement,References>();
		this.followers=new HashSet<Subject>();
		this.challenges=new ArrayList<Implication>();
		this.trash=new ArrayList<Statement>();
		this.blockSet=new HashSet<User>();
		this.removeIndex=new HashMap<Statement, Statements>();
		this.assumptionToChallengeIndex=new HashMap<Statement,Implication>();
		this.conclusionToChallengeIndex=new HashMap<Statement,Implication>();
		
		
	}

	public String toString(Language lang){
		
		String string=null;
		
		switch(lang){
		
		case English: string="Subject "+this.name;
		break;
		}
		
		
		return string;
		
	}
	@Override
	public Constructor getConstructorForFactory() {
		
		Constructor con=null;
		// TODO Auto-generated method stub
		try {
			con=Subject.class.getConstructor(User.class);
		} catch (NoSuchMethodException | SecurityException e) {
			
			
			LOG.severe("Subject couldn't give his constructor for Factory!");
		}
		
		return con;
	}
	
	
	//***************************Helping functions:
	
	
	
	public boolean agree(Statement statement){
		
		return this.believes.containsKey(statement);
	}
	
	public boolean disagree(Statement statement){
		
		return this.believes.containsKey(statement.getNegation());
	}
	
	public boolean blockes(Statement statement){
		
		return this.trash.contains(statement);
	}
	
	public boolean blockes(User user){
		
		return this.blockSet.contains(user);
	}
	
	public boolean hasOpinionAbout(Statement statement){
		
		return this.agree(statement)||this.disagree(statement)||this.blockes(statement);
		
		
	}
	
	public boolean ableToAgreeWith(Statement statement){
		
		boolean able=true;
		
		if(this.blockes(statement)){
			
			able=false;
		}else{
			
			if(this.disagree(statement)){
				
				able=false;
			}
		}
		return able;
	}

	public boolean hasFollowers() {
		// TODO Auto-generated method stub
		return followers.size()>0;
	}
	
	
	
	
	

}
	

	

	
	
	
	
	


