package entities;

import java.lang.reflect.Constructor;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.ConsoleHandler;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.persistence.Basic;
import javax.persistence.CascadeType;
import javax.persistence.Entity;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.TypedQuery;

import definitions.Findable;
import definitions.Language;

@Entity
@Table
public class User extends LogicalObject implements Findable{
	
	transient private static  final Logger LOG=Logger.getLogger(User.class.getName());
	static{
		
			LOG.setLevel(Level.ALL);
			ConsoleHandler console=new ConsoleHandler();
			console.setLevel(Level.ALL);
		
	}
	
	private String userName;
	
	private String password;
	
	private Language language;
	
	private String email;
	
	@Basic
	public String getEmail() {
		return email;
	}

	public void setEmail(String email) {
		
		this.email = email;
	}
	
	private List<Subject> subjects;
	
	private List<Statement>SandBox;

	@OneToMany
	public List<Statement> getSandBox() {
		return SandBox;
	}

	public void setSandBox(List<Statement> sandBox) {
		SandBox = sandBox;
	}

	@OneToMany(cascade={CascadeType.PERSIST,CascadeType.MERGE})
	public List<Subject> getSubjects() {
		return subjects;
	}

	public void setSubjects(List<Subject> subjects) {
		this.subjects = subjects;
	}

	public User(String loginString, String password){
		this();
		
		this.userName=loginString;
		this.password=password;
		
		
	}
	
	public User() {
		
		this.subjects=new ArrayList<Subject>();
		this.SandBox=new ArrayList<Statement>();
		
		
	}

	@Basic
	public String getUserName() {
		return userName;
	}

	public void setUserName(String userName) {
		this.userName = userName;
	}

	@Basic
	public String getPassword() {
		return password;
	}

	public void setPassword(String password) {
		this.password = password;
	}
	
	@Basic
	public Language getLanguage() {
		return language;
	}

	public void setLanguage(Language language) {
		this.language = language;
	}
	
	
public String toString(Language lang){
		
		String string=null;
		
		switch(lang){
		
		case English: string="User "+userName+" ("+password+")";
		break;
		}
		
		
		return string;
		
	}
	@Override
	public Constructor getConstructorForFactory() {
		
		Constructor con=null;
		// TODO Auto-generated method stub
		try {
			con=User.class.getConstructor(String.class,String.class);
		} catch (NoSuchMethodException | SecurityException e) {
			
			
			LOG.severe("Subject couldn't give his constructor for Factory!");
		}
		
		return con;
	}

	@Override
	public String getFindingQLString() {
		
		
		// TODO Auto-generated method stub
		return "select user from User user where user.userName=:loginString or user.email=:loginString";
	}

	@Override
	public TypedQuery<LogicalObject> prepareQuery(
			TypedQuery<LogicalObject> query, Object... args) {
		
		String loginString=(String) args[0];
		// TODO Auto-generated method stub
		return query.setParameter("loginString",loginString);
	}
	



	

}
